@@ -16,29 +16,36 @@ mkdir -p package-zip
 cp texstudio.exe package-zip/
 cd package-zip
 #cp /mingw64/bin/libicudt68.dll /mingw64/bin/icudt68.dll
-windeployqt-qt6 texstudio.exe
+# copy of translations apparently broken currently
+windeployqt-qt6  --no-qml --qmlcore --no-translations texstudio.exe
 echo "copy dlls"
 #ldd texstudio.exe | awk '{print $3}'| grep ming | xargs -I{} cp -u {} .
 ldd texstudio.exe | awk '{print $3}'| grep ucrt64 | xargs -I{} cp -u {} .
 # force ssl/crypto copy
-ldd texstudio.exe | awk '{print $3}'| grep libcrypto | xargs -I{} cp -u {} .
-ldd texstudio.exe | awk '{print $3}'| grep libssl | xargs -I{} cp -u {} .
+#ldd texstudio.exe | awk '{print $3}'| grep libcrypto | xargs -I{} cp -u {} .
+#ldd texstudio.exe | awk '{print $3}'| grep libssl | xargs -I{} cp -u {} .
 # check copied dlls
 echo "check dlls"
 ldd texstudio.exe
 cd ..
 echo "copy directories"
 mkdir -p ./package-zip/translations
-cp -r ../translation/* ./package-zip/translations
+cp -r ../translation/texstudio*.qm ./package-zip/translations
+cp -r $MSYSTEM_PREFIX/share/qt6/translations/qt_*.qm ./package-zip/translations
+cp -r $MSYSTEM_PREFIX/share/qt6/translations/qtbase_*.qm ./package-zip/translations
+cp -r $MSYSTEM_PREFIX/share/qt6/translations/qtmultimedia_*.qm ./package-zip/translations
 cp -r ../templates package-zip
-cp -r ../utilities/manual package-zip/help
+mkdir -p ./package-zip/help
+cp -r ../utilities/manual/build package-zip/help
+cp ../utilities/manual/source/CHANGELOG.md package-zip/help
 cp ../utilities/latex2e.css package-zip/help
+cp ../utilities/list.png package-zip/help
 cp ../utilities/latex2e.html package-zip/help
 cp -r ../utilities/dictionaries package-zip/dictionaries
 cp -r ../utilities/TexTablet package-zip/TexTablet
 mkdir -p package-zip/share
 cp -r package-zip/tls .
-cp -r ../travis-ci/mxe/fonts package-zip/share/fonts
+cp -r ../utilities/fonts package-zip/share/fonts
 # poppler data
 cd package-zip
 cp -r $MSYSTEM_PREFIX/share/poppler share
