@@ -9,6 +9,7 @@
 #include "latexparser/latexparser.h"
 #include "latexpackage.h"
 #include "testutil.h"
+#include "configmanager.h"
 #include <QtTest/QtTest>
 
 // shortcuts and semantic types
@@ -156,7 +157,7 @@ void LatexParsingTest::test_latexLexing_data() {
                                   << (Levels() << 0 << 1 << 1);
     QTest::newRow("text command with nested braces") << "\\textbf{text {abc}}"
                                   << (TTypes() << T::command << T::braces << T::word << T::braces << T::word)
-                                  << (STypes() << T::none << T::text << T::text << T::text << T::text)
+                                  << (STypes() << T::none << T::text << T::text << T::none << T::none)
                                   << (Starts() << 0 << 7 << 8 << 13 << 14)
                                   << (Length() << 7 << 12 << 4 << 5 << 3)
                                   << (Levels() << 0 << 1 << 1 << 1 << 1);
@@ -220,14 +221,19 @@ void LatexParsingTest::test_latexLexing_data() {
                                          << (Starts() << 0 << 11 << 12 << 23 )
                                          << (Length() << 11 << 12 << 10 << 6 )
                                          << (Levels() << 0 << 1 << 1 << 1 );
-    /*
+    QTest::newRow("newcommand multiline definition")
+        << "\\newcommand{\\paragraph}{test\nasdasd\nasd}"
+        << (TTypes() << T::command << T::braces << T::def << T::openBrace << T::closeBrace )
+        << (STypes() << T::none << T::def << T::none << T::definition << T::definition)
+        << (Starts() << 0 << 11 << 12 << 23 <<3)
+        << (Length() << 11 << 12 << 10 << 5 <<1)
+        << (Levels() << 0 << 1 << 1 << 1 <<1);
     QTest::newRow("newcommand nobrace") << "\\newcommand\\foo{test}"
-                                        << (TTypes() << T::command << T::def << T::braces << T::word)
-                                        << (STypes() << T::none << T::none << T::definition << T::definition)
-                                        << (Starts() << 0 << 11 << 15 << 16)
-                                        << (Length() << 11 << 4 << 6 << 4)
-                                        << (Levels() << 0 << 0 << 1 << 1);
-    */
+                                        << (TTypes() << T::command << T::command << T::braces)
+                                        << (STypes() << T::none << T::def << T::definition)
+                                        << (Starts() << 0 << 11 << 15)
+                                        << (Length() << 11 << 4 << 6)
+                                        << (Levels() << 0 << 1 << 1);
     QTest::newRow("documentclass command") << "\\documentclass{text}"
                                            << (TTypes() << T::command << T::braces << T::documentclass)
                                            << (STypes() << T::none << T::documentclass << T::none)
@@ -264,6 +270,43 @@ void LatexParsingTest::test_latexLexing_data() {
                                                   << (Starts() << 0 << 16 << 17 << 21 << 25 << 30 << 31)
                                                   << (Length() << 16 << 14 << 3 << 3 << 4 << 6 << 4)
                                                   << (Levels() << 0 << 1 << 1 << 1 << 2 << 1 << 1);
+    QTest::newRow("graphics command with keyval, multi-line") << "\\includegraphics[opt,\nopt=text]{file}"
+                                                  << (TTypes() << T::command << T::openSquare << T::keyVal_key << T::keyVal_key << T::word << T::closeSquareBracket << T::braces << T::imagefile)
+                                                  << (STypes() << T::none << T::keyValArg << T::none << T::none << T::keyVal_val << T::keyValArg << T::imagefile << T::none)
+                                                  << (Starts() << 0 << 16 << 17 << 0 << 4 << 8 << 9 << 10)
+                                                  << (Length() << 16 << 5 << 3 << 3 << 4 << 1 << 6 << 4)
+                                                  << (Levels() << 0 << 1 << 1 << 1 << 2 << 1 << 1 << 1);
+    QTest::newRow("graphics command with keyval in braces") << "\\includegraphics[opt={text}]{file}"
+                                                  << (TTypes() << T::command << T::squareBracket << T::keyVal_key << T::braces << T::word  << T::braces << T::imagefile)
+                                                  << (STypes() << T::none << T::keyValArg << T::none << T::keyVal_val << T::keyVal_val << T::imagefile << T::none)
+                                                  << (Starts() << 0  << 16 << 17 << 21 << 22 << 28 << 29)
+                                                  << (Length() << 16 << 12 << 3  <<  6 << 4  << 6  << 4)
+                                                  << (Levels() << 0  << 1  << 1  << 2  << 2  << 1  << 1);
+    QTest::newRow("graphics command with keyval in braces, multiline") << "\\includegraphics[opt={\ntext\n}]{file}"
+                                                  << (TTypes() << T::command << T::openSquare << T::keyVal_key << T::openBrace << T::word  << T::closeBrace << T::closeSquareBracket << T::braces << T::imagefile)
+                                                  << (STypes() << T::none << T::keyValArg << T::none << T::keyVal_val << T::keyVal_val << T::keyVal_val << T::keyValArg << T::imagefile << T::none)
+                                                  << (Starts() << 0  << 16 << 17 << 21 << 0 << 0 << 1 << 2 << 3 )
+                                                  << (Length() << 16 << 6  << 3  <<  1 << 4 << 1 << 1 << 6 << 4 )
+                                                  << (Levels() << 0  << 1  << 1  << 2  << 3 << 3 << 2 << 1 << 1 );
+    QTest::newRow("listings command with defined keyval argument") << "\\lstdefinelanguage{Excel}{morekeywords={ab$c}}"
+                                                                   << (TTypes() << T::command << T::braces     << T::word       << T::braces    << T::keyVal_key << T::braces )
+                                                                   << (STypes() << T::none    << T::generalArg << T::generalArg << T::keyValArg << T::none       << T::definition)
+                                                                   << (Starts() << 0  << 18 << 19 << 25 << 26 << 39)
+                                                                   << (Length() << 18 <<  7 <<  5 << 21 << 12 <<  6)
+                                                                   << (Levels() << 0  <<  1 <<  1 <<  1 <<  1 <<  3);
+    QTest::newRow("listings command with defined keyval argument2") << "\\lstdefinelanguage{Excel}{morekeywords={ab$c,sd&f}}"
+                                                                   << (TTypes() << T::command << T::braces     << T::word       << T::braces    << T::keyVal_key << T::braces )
+                                                                   << (STypes() << T::none    << T::generalArg << T::generalArg << T::keyValArg << T::none       << T::definition)
+                                                                   << (Starts() << 0  << 18 << 19 << 25 << 26 << 39)
+                                                                   << (Length() << 18 <<  7 <<  5 << 26 << 12 << 11)
+                                                                   << (Levels() << 0  <<  1 <<  1 <<  1 <<  1 <<  3);
+    QTest::newRow("listings command with defined keyval argument multi line")
+        << "\\lstdefinelanguage{Excel}{morekeywords={ab$c,\nsd&f\n}} test"
+        << (TTypes() << T::command << T::braces     << T::word       << T::openBrace << T::keyVal_key << T::openBrace << T::closeBrace << T::closeBrace << T::word)
+        << (STypes() << T::none    << T::generalArg << T::generalArg << T::keyValArg << T::none       << T::definition<< T::definition << T::keyValArg  << T::none)
+        << (Starts() << 0  << 18 << 19 << 25 << 26 << 39 << 0 << 1 << 3)
+        << (Length() << 18 <<  7 <<  5 << 20 << 12 <<  6 << 1 << 1 << 4)
+        << (Levels() << 0  <<  1 <<  1 <<  1 <<  1 <<  3 << 3 << 2 << 0);
     QTest::newRow("include command") << "\\include{text dsf}"
                                      << (TTypes() << T::command << T::braces << T::file)
                                      << (STypes() << T::none << T::file << T::none)
@@ -294,12 +337,56 @@ void LatexParsingTest::test_latexLexing_data() {
                                       << (Starts() << 0 << 1 << 7 << 12)
                                       << (Length() << 1 << 5 << 5 << 1)  // note: the brackets are ignored, they are not part of the command
                                       << (Levels() << 0 << 0 << 0 << 0);
+    QTest::newRow("unused argument in text command") << "\\textbf{\\textbf} text"
+                                     << (TTypes() << T::command << T::braces << T::command << T::word)
+                                     << (STypes() << T::none << T::text << T::text << T::none)
+                                     << (Starts() << 0 << 7 << 8 <<17)
+                                     << (Length() << 7 << 9 << 7 << 4)
+                                     << (Levels() << 0 << 1 << 1 << 0);
+    QTest::newRow("positional optional arguments") << "\\yagding{char}[color]"
+                                                  << (TTypes() << T::command << T::braces << T::word << T::squareBracket << T::word)
+                                                  << (STypes() << T::none << T::generalArg << T::generalArg << T::color << T::color)
+                                                  << (Starts() << 0 << 8 << 9 << 14 << 15)
+                                                  << (Length() << 8 << 6 << 4 << 7 << 5)
+                                                  << (Levels() << 0 << 1 << 1 << 1 << 1);
+    QTest::newRow("positional optional arguments2") << "\\yagding[test]{char}[color]"
+                                                   << (TTypes() << T::command << T::squareBracket << T::keyVal_key << T::braces << T::word << T::squareBracket << T::word)
+                                                    << (STypes() << T::none << T::keyValArg << T::none << T::generalArg << T::generalArg << T::color << T::color)
+                                                    << (Starts() << 0 << 8 << 9 << 14 << 15 << 20 << 21)
+                                                    << (Length() << 8 << 6 << 4 << 6 << 4 << 7 << 5)
+                                                    << (Levels() << 0 << 1 << 1 << 1 << 1 << 1 << 1);
+    QTest::newRow("positional optional arguments3") << "\\yagding[test]{char} color"
+                                                    << (TTypes() << T::command << T::squareBracket << T::keyVal_key << T::braces << T::word << T::word)
+                                                    << (STypes() << T::none << T::keyValArg << T::none << T::generalArg << T::generalArg << T::none)
+                                                    << (Starts() << 0 << 8 << 9 << 14 << 15 <<  21)
+                                                    << (Length() << 8 << 6 << 4 << 6 << 4 << 5)
+                                                    << (Levels() << 0 << 1 << 1 << 1 << 1 << 0);
+    QTest::newRow("default overlay") << "\\againframe[<test>]"
+                                                   << (TTypes() << T::command << T::squareBracket << T::overlay)
+                                                   << (STypes() << T::none << T::overlay << T::none)
+                                                   << (Starts() << 0 << 11 << 12)
+                                                   << (Length() << 11 << 8 << 6)
+                                                   << (Levels() << 0 << 1 << 1 );
+    QTest::newRow("option") << "\\againframe[test]"
+                                     << (TTypes() << T::command << T::squareBracket << T::keyVal_key)
+                                     << (STypes() << T::none << T::keyValArg << T::none)
+                                     << (Starts() << 0 << 11 << 12)
+                                     << (Length() << 11 << 6 << 4)
+                                     << (Levels() << 0 << 1 << 1 );
+
 }
 
 void LatexParsingTest::test_latexLexing() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
+    LatexPackage pkg_tex = loadCwlFile("tex.cwl");
+    lp->commandDefs.unite(pkg_tex.commandDescriptions);
+    LatexPackage pkg_listings = loadCwlFile("txs-test.cwl");
+    lp->commandDefs.unite(pkg_listings.commandDescriptions);
+    LatexPackage pkg_yagu = loadCwlFile("yagusylo.cwl");
+    lp->commandDefs.unite(pkg_yagu.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(TTypes, types);
     QFETCH(STypes, subtypes);
@@ -324,10 +411,6 @@ void LatexParsingTest::test_latexLexing() {
         QDocumentLineHandle *dlh = doc->line(i).handle();
         tl.append(dlh->getCookieLocked(QDocumentLine::LEXER_COOKIE).value<TokenList>());
     }
-	qDebug() << "XXX";
-	for(int i=0; i<tl.length(); i++){
-		qDebug() << tl.at(i);
-	}
 
 	for(int i=0; i<tl.length(); i++){
         Token tk = tl.at(i);
@@ -402,9 +485,10 @@ void LatexParsingTest::test_findCommandWithArgsFromTL_data() {
 }
 
 void LatexParsingTest::test_findCommandWithArgsFromTL() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(TTypes, types);
     QFETCH(STypes, subtypes);
@@ -449,7 +533,7 @@ void LatexParsingTest::test_getArg_data() {
     QTest::addColumn<QStringList>("desiredResults");
 
 
-    /*QTest::newRow("newcommand command") << "\\newcommand{text}{test}"
+    QTest::newRow("newcommand command") << "\\newcommand{text}{test}"
                                         << (STypes() <<T::def << T::definition)
                                         << (QStringList() <<"text" <<"test");
     QTest::newRow("newcommand command2") << "\\newcommand{\\ext}{test}"
@@ -465,7 +549,7 @@ void LatexParsingTest::test_getArg_data() {
     QTest::newRow("text command, embedded") << "\\textbf{te\\textit{xt} bg}"
                                             << (STypes() <<T::text) << (QStringList() <<"te\\textit{xt} bg");
     QTest::newRow("text command, embedded ,open") << "\\textbf{  te\\textit{xt} bg"
-                                            <<   (STypes() <<T::text) << (QStringList() <<"  te\\textit{xt} bg");*/
+                                            <<   (STypes() <<T::text) << (QStringList() <<"  te\\textit{xt} bg");
     QTest::newRow("text command, multi-line") << "\\textbf{  te\n bg}"
                                             <<   (STypes() <<T::text) << (QStringList() <<"  te bg");
     QTest::newRow("text command, multi-line, with comment") << "\\textbf{  te%abc\n bg}"
@@ -486,9 +570,10 @@ void LatexParsingTest::test_getArg_data() {
 }
 
 void LatexParsingTest::test_getArg() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(STypes, types);
     QFETCH(QStringList, desiredResults);
@@ -633,9 +718,12 @@ void LatexParsingTest::test_getArg2_data() {
 }
 
 void LatexParsingTest::test_getArg2() {
-    LatexParser lp = LatexParser::getInstance();
+    int storeRUNAWAYLIMIT=ConfigManager::RUNAWAYLIMIT;
+    ConfigManager::RUNAWAYLIMIT=30;
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(ATypes, types);
     QFETCH(QList<int>, nr);
@@ -665,6 +753,79 @@ void LatexParsingTest::test_getArg2() {
         QCOMPARE(result, desiredResults.at(i));
     }
     delete doc;
+    ConfigManager::RUNAWAYLIMIT=storeRUNAWAYLIMIT;
+}
+
+void LatexParsingTest::test_getArgContent_data()
+{
+    QTest::addColumn<QString>("lines");
+    QTest::addColumn<STypes>("types");
+    QTest::addColumn<STypes>("subtypes");
+    QTest::addColumn<int>("pos");
+    QTest::addColumn<int>("level");
+
+    QTest::newRow("newcommand command") << "\\newcommand{text}{test}"
+                                        << (TTypes() <<T::braces << T::def << T::braces)
+                                        << (STypes() <<T::def << T::none << T::definition)
+                                        << 0 << 1;
+    QTest::newRow("newcommand command2") << "\\newcommand{\\ext}{test}"
+                                         <<  (STypes() <<T::braces << T::def << T::braces)
+                                         <<  (STypes() <<T::def << T::none << T::definition)
+                                         << 0 << 1;
+    QTest::newRow("newcommand command, no braces") << "\\newcommand text {test}"
+                                                   <<  (STypes() <<T::word << T::braces)
+                                                   <<  (STypes() <<T::def << T::definition)
+                                                   << 0 << 1;
+    QTest::newRow("newcommand command, no braces2") << "\\newcommand text test"
+                                                    <<  (TTypes() << T::word << T::word)
+                                                    <<  (STypes() << T::def << T::definition)
+                                                    << 0 << 1;
+    QTest::newRow("newcommand command with trail") << "\\newcommand{text}{test} abc"
+                                        << (TTypes() <<T::braces << T::def << T::braces)
+                                        << (STypes() <<T::def << T::none << T::definition)
+                                        << 0 << 1;
+    QTest::newRow("newcommand command with trailing braces") << "\\newcommand{text}{test} {abc}"
+                                                   << (TTypes() <<T::braces << T::def << T::braces)
+                                                   << (STypes() <<T::def << T::none << T::definition)
+                                                   << 0 << 1;
+}
+
+void LatexParsingTest::test_getArgContent()
+{
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
+    LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
+    QFETCH(QString,lines);
+    QFETCH(TTypes, types);
+    QFETCH(STypes, subtypes);
+    QFETCH(int, pos);
+    QFETCH(int, level);
+
+    QDocument *doc = new QDocument();
+    doc->setText(lines, false);
+    for(int i=0; i<doc->lines(); i++){
+        QDocumentLineHandle *dlh = doc->line(i).handle();
+        Parsing::simpleLexLatexLine(dlh);
+    }
+    TokenStack stack;
+    CommandStack commandStack;
+    for(int i=0; i<doc->lines(); i++){
+        QDocumentLineHandle *dlh = doc->line(i).handle();
+        Parsing::latexDetermineContexts2(dlh, stack, commandStack, lp);
+    }
+    QDocumentLineHandle *dlh = doc->line(0).handle();
+    TokenList tl= dlh->getCookieLocked(QDocumentLine::LEXER_COOKIE).value<TokenList >();
+    // first token is command
+    Token tkCmd;
+    TokenList args;
+    TokenList result=Parsing::getArgContent(tl, pos,level);
+    QCOMPARE(result.length(), types.length());
+    for(int i=0;i<result.length();++i){
+        QCOMPARE(result.at(i).type, types.at(i));
+        QCOMPARE(result.at(i).subtype, subtypes.at(i));
+    }
+    delete doc;
 }
 
 void LatexParsingTest::test_getTokenAtCol_data() {
@@ -699,9 +860,10 @@ void LatexParsingTest::test_getTokenAtCol_data() {
 }
 
 void LatexParsingTest::test_getTokenAtCol() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(QList<int>, nr);
     QFETCH(TTypes, desiredResults);
@@ -825,9 +987,10 @@ void LatexParsingTest::test_getCommandFromToken_data() {
 }
 
 void LatexParsingTest::test_getCommandFromToken() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(int, nr);
     QFETCH(int, lineNr);
@@ -883,8 +1046,10 @@ void LatexParsingTest::test_getContext_data() {
                             << (STypes() << T::none << T::keyValArg<<T::none);
     QTest::newRow("command with keyval2") << "\\includegraphics[width=4cm]{abc}"
                             << 23
-                            << (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::width)
-                            << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
+                            << (TTypes() << T::command << T::squareBracket)
+                            << (STypes() << T::none << T::keyValArg);
+                            //<< (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::width) // this may be the desired outcome
+                            //<< (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
     QTest::newRow("command with keyval3") << "\\includegraphics[width=4cm]{abc}"
                             << 24
                             << (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::width)
@@ -893,14 +1058,40 @@ void LatexParsingTest::test_getContext_data() {
                             << 28
                             << (TTypes() << T::command << T::squareBracket<<T::keyVal_key)
                             << (STypes() << T::none << T::keyValArg<<T::none);
+    QTest::newRow("command with empty keyval") << "\\includegraphics[width=]{abc}"  // #4017
+                            << 23
+                            << (TTypes() << T::command << T::squareBracket<<T::keyVal_key<<T::keyVal_val)
+                            << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
+    QTest::newRow("command with keyval as defined argument") << "\\lstdefinelanguage{Excel}{morekeywords={ab$c}}"
+                                          << 41
+                                                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::braces)
+                                                             << (STypes() << T::none << T::keyValArg<<T::none<<T::definition);
+    QTest::newRow("command with keyval as label") << "\\lstdefinelanguage{Excel}{label=test}"
+                                                             << 35
+                                                             << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::label)
+                                                             << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val);
+    QTest::newRow("following command") << "bummerang\\text" // command after word, #3967
+                            << 9
+                            << (TTypes() << T::word)
+                            << (STypes() << T::none);
+    // stacked commands, keyval with arguments which are comma separated (#4074)
+    QTest::newRow("command with keyval with command with comma separated arguments")
+        << "\\mycommand{note=\\cite{abc}}"
+        << 23
+        << (TTypes() << T::command << T::braces<<T::keyVal_key<<T::command<<T::braces<<T::bibItem)
+        << (STypes() << T::none << T::keyValArg<<T::none<<T::keyVal_val<<T::bibItem<<T::none);
+
 
 
 }
 
 void LatexParsingTest::test_getContext() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
+    LatexPackage pkg_listings = loadCwlFile("txs-test.cwl");
+    lp->commandDefs.unite(pkg_listings.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(int, nr);
     QFETCH(TTypes, desiredResults);
@@ -924,9 +1115,13 @@ void LatexParsingTest::test_getContext() {
     TokenStack result = Parsing::getContext(dlh,nr);
 
     for(int k=0;k<result.size();k++){
+        if(k>=desiredResults.size()){
+            continue;
+        }
         QVERIFY2(result.at(k).type==desiredResults.at(k), QString("incorrect type at index %1:%2").arg(k).arg(lines).toLatin1());
         QVERIFY2(result.at(k).subtype==types.at(k), QString("incorrect subtype at index %1:%2").arg(k).arg(lines).toLatin1());
     }
+    QVERIFY2(result.size()==desiredResults.size(), QString("incorrect stacksize: %1:%2").arg(result.size()).arg(desiredResults.size()).toLatin1());
 
     delete doc;
 }
@@ -934,41 +1129,42 @@ void LatexParsingTest::test_getContext() {
 void LatexParsingTest::test_getCompleterContext_data() {
     QTest::addColumn<QString>("lines");
     QTest::addColumn<int >("nr");
-    QTest::addColumn<int >("desiredResult");
+    QTest::addColumn<EnumsTokenType::TokenType >("desiredResult");
 
     QTest::newRow("simple") << "bummerang"
                             << 2
-                            <<  0;
+                            <<  EnumsTokenType::word;
     QTest::newRow("command") << "\\section{abc}"
                             << 10
-                            << 0;
+                            << EnumsTokenType::title;
     QTest::newRow("command without braces") << "\\section abc"
                             << 10
-                            << 0;
+                            << EnumsTokenType::title;
     QTest::newRow("command with optional arg") << "\\section[fds]{abc}"
                             << 10
-                            << 0;
+                            << EnumsTokenType::shorttitle;
     QTest::newRow("command with keyval") << "\\includegraphics[width=4cm]{abc}"
                             << 18
-                            << 0;
+                            << EnumsTokenType::keyVal_key;
     QTest::newRow("command with keyval2") << "\\includegraphics[width=4cm]{abc}"
                             << 24
-                            << 512;
+                            << EnumsTokenType::width;
     QTest::newRow("length") << "\\hspace{4cm}"
                             << 9
-                            << 512;
+                            << EnumsTokenType::width;
 
 
 
 }
 
 void LatexParsingTest::test_getCompleterContext() {
-    LatexParser lp = LatexParser::getInstance();
+    QSharedPointer<LatexParser> lp = QSharedPointer<LatexParser>::create();
+    *lp=LatexParser::getInstance();
     LatexPackage pkg_graphics = loadCwlFile("graphicx.cwl");
-    lp.commandDefs.unite(pkg_graphics.commandDescriptions);
+    lp->commandDefs.unite(pkg_graphics.commandDescriptions);
     QFETCH(QString,lines);
     QFETCH(int, nr);
-    QFETCH(int, desiredResult);
+    QFETCH(EnumsTokenType::TokenType, desiredResult);
 
 
     QDocument *doc = new QDocument();
@@ -986,7 +1182,7 @@ void LatexParsingTest::test_getCompleterContext() {
     QDocumentLineHandle *dlh = doc->line(0).handle();
     //TokenList tl= dlh->getCookieLocked(QDocumentLine::LEXER_COOKIE).value<TokenList >();
 
-    int result = Parsing::getCompleterContext(dlh, nr);
+    EnumsTokenType::TokenType result = Parsing::getCompleterContext(dlh, nr);
 
     QCOMPARE(result, desiredResult);
 
