@@ -14,6 +14,7 @@
 #ifndef Header_TexStudio
 #define Header_TexStudio
 
+#include "aichatassistant.h"
 #include "mostQtHeaders.h"
 
 #include "bibtexparser.h"
@@ -26,6 +27,7 @@
 #include "latexdocument.h"
 #include "latexeditorview.h"
 #include "latexcompleter.h"
+#include "usermacro.h"
 #include "xmltagslistwidget.h"
 #include "spellerdialog.h"
 #include "textanalysis.h"
@@ -50,6 +52,7 @@
 #include "help.h"
 
 #include <QProgressDialog>
+#include <QFileSystemModel>
 
 /*!
  * \file texstudio.h
@@ -86,10 +89,11 @@ public:
 
     Q_INVOKABLE void runInternalCommand(const QString &cmd, const QString &master, const QString &options);
 
+    friend class TexStudioTest;
+
 public slots:
-	LatexEditorView *load(const QString &f , bool asProject = false, bool hidden = false, bool recheck = true, bool dontAsk = false);
+	LatexEditorView *load(const QString &f , bool asProject = false, bool recheck = true, bool dontAsk = false);
     void executeCommandLine(const QStringList &args, bool realCmdLine);
-	void hideSplash(); ///< hide splash screen
 	void startupCompleted();
 	void onOtherInstanceMessage(const QString &);  ///< For messages for the single instance
 	void fuzzCursorHistory();
@@ -101,18 +105,21 @@ protected:
 	//these are just wrappers around configManager so we don't have to type so much (todo??? move them to configmanager.h and use a singleton design???)
 	Q_INVOKABLE inline QMenu *newManagedMenu(const QString &id, const QString &text) { return configManager.newManagedMenu(id, text); }
 	Q_INVOKABLE inline QMenu *newManagedMenu(QMenu *menu, const QString &id, const QString &text) { return configManager.newManagedMenu(menu, id, text); }
-    Q_INVOKABLE QAction *newManagedAction(QWidget *menu, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
+	Q_INVOKABLE QAction *newManagedAction(QWidget *menu, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
 	Q_INVOKABLE QAction *newManagedAction(QWidget *menu, const QString &id, const QString &text, const char *slotName, const QList<QKeySequence> &shortCuts, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
-    Q_INVOKABLE QAction *newManagedEditorAction(QWidget *menu, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
+	Q_INVOKABLE QAction *newManagedEditorAction(QWidget *menu, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
 	Q_INVOKABLE QAction *newManagedEditorAction(QWidget *menu, const QString &id, const QString &text, const char *slotName, const QList<QKeySequence> &shortCuts, const QString &iconFile = "", const QList<QVariant> &args = QList<QVariant>());
 	Q_INVOKABLE inline QAction *newManagedAction(QWidget *menu, const QString &id, QAction *act) { return configManager.newManagedAction(menu, id, act); }
 	Q_INVOKABLE inline QMenu *getManagedMenu(const QString &id) { return configManager.getManagedMenu(id); }
 	Q_INVOKABLE inline QAction *getManagedAction(const QString &id) { return configManager.getManagedAction(id); }
 	Q_INVOKABLE inline QList<QAction *> getManagedActions(const QStringList &ids, const QString &commonPrefix = "") { return configManager.getManagedActions(ids, commonPrefix); }
-    Q_INVOKABLE QAction *insertManagedAction(QAction *before, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "");
+	Q_INVOKABLE QAction *insertManagedAction(QAction *before, const QString &id, const QString &text, const char *slotName = nullptr, const QKeySequence &shortCut = 0, const QString &iconFile = "");
+	Q_INVOKABLE void loadManagedMenu(const QString &fn);
+
+	Q_INVOKABLE void setupToolBars();
 
 	void addTagList(const QString &id, const QString &iconName, const QString &text, const QString &tagFile);
-    void addMacrosAsTagList();
+	void addMacrosAsTagList();
 
 private slots:
 	void updateToolBarMenu(const QString &menuName);
@@ -123,14 +130,17 @@ private:
 	void generateAddtionalTranslations();
 	void setupMenus();
 	void setupDockWidgets();
-	void setupToolBars();
 	void createStatusBar();
 	bool activateEditorForFile(QString f, bool checkTemporaryNames = false, bool setFocus = true);
 	bool saveAllFilesForClosing(); ///< checks for unsaved files and asks the user if they should be saved
-    bool saveFilesForClosing(const QList<LatexDocument *> &documentList); ///< checks for unsaved files and asks the user if they should be saved
+    bool saveFilesForClosing(QList<LatexDocument *> &documentList); ///< checks for unsaved files and asks the user if they should be saved
 	void closeAllFiles();
 	bool canCloseNow(bool saveSettings = true); ///< asks the user and close all files, and prepares to exit txs
 	void closeEvent(QCloseEvent *e);
+    void setStructureSectionIcons();
+    void updateStatusBarIcons();
+    void updatePDFIcons();
+	void setCheckedPreviewModeAction();
 
 	void updateUserMacros(bool updateMenu = true);
 
@@ -146,16 +156,20 @@ private:
 
 	//gui
 	Editors *editors;
-	QSplitter *sidePanelSplitter;
 	QSplitter *mainHSplitter;
 	QSplitter *centralVSplitter;
 	QFrame *centralFrame;
 	QToolBar *centralToolBar;
-	CustomWidgetList *leftPanel;
-	TitledPanel *sidePanel;
+    QDockWidget *m_firstDockWidget = nullptr;
+    QAction *m_toggleDocksAction;
 	SymbolListModel *symbolListModel;
 	SymbolWidget *symbolWidget;
+    QTreeView *fileView; ///< file explorer in docks
+    QFileSystemModel *fileExplorerModel = nullptr;
 	QString hiddenLeftPanelWidgets;
+    QString docksToBeRaised; ///< when restoring docks from hidden sidepanel, these docks should be raised. Dock names are separated by "|"
+    QMap<QString, QString> m_dockIcons;
+    QList<QDockWidget*> m_docksOrder;
 
     //StructureTreeView *structureTreeView;
     QTreeWidget *structureTreeWidget;
@@ -227,16 +241,19 @@ private:
 	void updateUserToolMenu();
 	void linkToEditorSlot(QAction *act, const char *slot, const QList<QVariant> &args);
 
-    bool parseStruct(StructureEntry* se, QVector<QTreeWidgetItem *> &rootVector, QSet<LatexDocument*> *visited=nullptr, QList<QTreeWidgetItem *> *todoList=nullptr, int currentColor=0);
-    void parseStructLocally(StructureEntry* se, QVector<QTreeWidgetItem *> &rootVector, QList<QTreeWidgetItem *> *todoList=nullptr, QList<QTreeWidgetItem *> *labelList=nullptr, QList<QTreeWidgetItem *> *magicList=nullptr, QList<QTreeWidgetItem *> *biblioList=nullptr);
+    bool parseStruct(LatexDocument *doc, QVector<QTreeWidgetItem *> &rootVector, QSet<LatexDocument*> *visited=nullptr, QList<QTreeWidgetItem *> *todoList=nullptr, int currentColor=0);
+    void parseStructLocally(LatexDocument* document, QVector<QTreeWidgetItem *> &rootVector, QList<QTreeWidgetItem *> *todoList=nullptr, QList<QTreeWidgetItem *> *labelList=nullptr, QList<QTreeWidgetItem *> *magicList=nullptr, QList<QTreeWidgetItem *> *biblioList=nullptr, QList<QTreeWidgetItem *> *blockList=nullptr);
 #ifndef QT_NO_DEBUG
     void checkForShortcutDuplicate();
 #endif
 private slots:
     void updateTOCs();
+    void updateAllTOCs();
 
     void updateTOC();
-    void updateCurrentPosInTOC(QTreeWidgetItem *root=nullptr,StructureEntry *old=nullptr,StructureEntry *selected=nullptr);
+    void updateCurrentPosInTOC(StructureEntry *old=nullptr, StructureEntry *selected=nullptr);
+    void updateCurrentPosInStructure(StructureEntry *old=nullptr, StructureEntry *selected=nullptr);
+    void updateCurrentPosInTOCHelper(QTreeWidgetItem *root=nullptr, StructureEntry *old=nullptr, StructureEntry *selected=nullptr, bool tocMode=true);
     void syncExpanded(QTreeWidgetItem *item);
     void syncCollapsed(QTreeWidgetItem *item);
     void editSectionCopy();
@@ -258,7 +275,7 @@ private slots:
     void collapseSubitems();
     StructureEntry *labelForStructureEntry(const StructureEntry *entry);
 
-    void updateStructureLocally();
+    void updateStructureLocally(bool updateAll=false);
     void customMenuStructure(const QPoint &pos);
     void createLabelFromAction();
 
@@ -319,7 +336,7 @@ private slots:
 	void fileUpdate(QString filename = "");
 	void fileUpdateCWD(QString filename = "");
 	void checkinAfterSave(QString filename, int checkIn = 0);
-	void checkin(QString fn, QString text = "txs auto checkin", bool blocking = false);
+    void checkin(QString fn, QString text = "txs auto checkin", bool push = false);
 	bool svnadd(QString fn, int stage = 0);
 	void svnUndo(bool redo = false);
 	void svnPatch(QEditor *ed, QString diff);
@@ -336,7 +353,7 @@ private slots:
 	void fileDiffMerge();
 	void declareConflictResolved();
 protected slots:
-    void openExternalFile(QString name, const QString &defaultExt = "tex", LatexDocument *doc = nullptr,bool relativeToCurrentDoc=false); // signaled by latexViewer to open specific file
+    LatexEditorView * openExternalFile(QString name, const QString &defaultExt = "tex", LatexDocument *doc = nullptr,bool relativeToCurrentDoc=false); // signaled by latexViewer to open specific file
     void openExternalFileFromAction();
 
 	void editUndo(); ///< undo changes in text editor
@@ -368,6 +385,7 @@ protected slots:
 	void editSpell();
 	void editThesaurus(int line = -1, int col = -1);
 	void editChangeLineEnding();
+	void setPreviewMode();
 	void editSetupEncoding();
 	void editInsertUnicode(); ///< open dialog to insert a unicode character
 	void editInsertRefToNextLabel(const QString &refCmd = "\\ref", bool backward = false);
@@ -380,16 +398,18 @@ protected slots:
 	void findWordRepetions();
 	void findNextWordRepetion();
 
-	void addDocToLoad(QString filename);
-
 	void LTErrorMessage(QString message);
 
     void paletteChanged(const QPalette &palette);
+#if (QT_VERSION >= 0x060500) && (defined( Q_OS_WIN )||defined( Q_OS_LINUX ))
+    void colorSchemeChanged(Qt::ColorScheme  colorScheme);
+#endif
 
 private slots:
 	void readSettings(bool reread = false); ///< read configured/default settings from ini
 	void saveSettings(const QString &configName = ""); ///< save all setting to ini
 	void restoreDefaultSettings(); ///< restore default settings, removing all changed values
+    void showSettings(); ///< show settings ini in editor for bug reporting
 
 protected slots:
 	void showMarkTooltipForLogMessage(QList<int> errors);
@@ -422,6 +442,8 @@ protected slots:
     void insertTextFromAction();
     void insertFromTagList(QListWidgetItem *item);
 	void insertBib();
+    void openFromExplorer(const QModelIndex &index);
+    void insertFromExplorer(bool visible);
 	void closeEnvironment();
 
 	void insertBibEntryFromAction();
@@ -454,7 +476,7 @@ protected slots:
     void addMagicBibliography();
     void addMagicProgram();
 
-	void quickTabular(); ///< start quick tabular wizard
+    void quickTabular(const QMimeData *d=nullptr); ///< start quick tabular wizard
 	void quickArray(); ///< start quick array wizard
 	void quickTabbing(); ///< start quick tabbing wizard
 	void quickLetter(); ///< start quick leter wizard
@@ -462,6 +484,7 @@ protected slots:
 	void quickBeamer(); ///< start quick beamer wizard
 	void quickGraphics(const QString &graphicsFile = QString()); ///< start quick graphics wizard
 	void quickMath(); ///< start quick math wizard
+    void aiChat(const QString queryText=""); ///< start ai chat assistant
 
 	bool checkProgramPermission(const QString &program, const QString &cmdId, LatexDocument *master);
 	void runInternalPdfViewer(const QFileInfo &master, const QString &options);
@@ -499,6 +522,7 @@ protected slots:
 
 	void webPublish();
 	void webPublishSource();
+	void convertToPlainText();
 	void analyseText();
 	void analyseTextFormDestroyed();
 	void generateRandomText();
@@ -536,6 +560,7 @@ protected slots:
 	void focusViewer();
 	void enlargeEmbeddedPDFViewer();
 	void shrinkEmbeddedPDFViewer(bool preserveConfig = false);
+	void setEnabledMenusEnlargeShrink(bool enabledEnlarge, bool enabledShrink);
 
 	void showStatusbar();
 	void viewCloseElement();
@@ -556,6 +581,7 @@ protected slots:
 
     void updateCompleter(LatexEditorView *edView = nullptr);
 	void completerNeedsUpdate();
+    void completerCommandsNeedsUpdate();
 	void needUpdatedCompleter();
 
 	void outputPageChanged(const QString &id);
@@ -596,7 +622,8 @@ protected slots:
 	void fileInConflictShowDiff();
 	void fileAutoReloading(QString fname);
 
-	void jumpToSearchResult(QDocument *doc, int lineNumber, const SearchQuery *query);
+	void jumpToSearchResult(LatexDocument *doc, int lineNumber, const SearchQuery *query);
+    void jumpToFileSearchResult(QString fn, int lineNumber, const SearchQuery *query);
 
 	void cursorPositionChanged();
 	void syncPDFViewer(QDocumentCursor cur, bool inForeground = true);
@@ -644,6 +671,9 @@ protected:
     bool eventFilter(QObject *obj, QEvent *event);
 
 	bool mCompleterNeedsUpdate;
+    bool mCompleterCommandsNeedsUpdate=false;
+    QSet<QString> mLoadedCWLFiles;
+    CodeSnippetList mCompleterWords;
 
 	int currentLine;
 	QBrush oldBackground;
@@ -655,12 +685,12 @@ protected:
 	UserMenuDialog *userMacroDialog;
 	QComboBox *cmbLog;
 
+    AIChatAssistant *aiChatDlg = nullptr;
+
 	QStringList m_columnCutBuffer;
 
 	QTimer autosaveTimer,previewDelayTimer,previewFullCompileDelayTimer;
 
-	bool completionBaseCommandsUpdated;
-
 	QSet<int> previewQueue;
 	LatexEditorView *previewQueueOwner;
 
@@ -681,11 +711,11 @@ protected:
 
 	void restoreBookmarks(LatexEditorView *edView);
 
+    QSet<QString> collectPotentialCompletionWords(const QDocument *doc, const QString &word) const;
+
 	bool completerPreview;
     QPixmapCache previewCache;
 
-	bool recheckLabels;
-
     bool rememberFollowFromScroll,enlargedViewer;
 
     std::set<QString> latexPackageList, currentPackageList;
@@ -694,6 +724,8 @@ protected:
 
     StructureEntry *currentSection;
 
+    bool mThesaurusWasStarted=false;
+
 public:
     Q_PROPERTY(QString clipboard READ clipboardText WRITE setClipboardText)
 	Q_INVOKABLE QString clipboardText(const QClipboard::Mode &mode = QClipboard::Clipboard) const;
@@ -714,6 +746,13 @@ public slots:
 	void slowOperationEnded();
 
 	void openBugsAndFeatures();
+    void maniplateDockingTabBars();
+    void addDock(const QString &name, const QString &iconName, const QString &title, QWidget *wgt);
+    void toggleDocks(bool visible);
+    void resetDocks();
+    void toggleDockVisibility();
+    void updateDockVisibility(bool visible);
+    bool checkDockSpread();
 
 signals:
 	void infoNewFile(); ///< signal that a new file has been generated. Used for scritps as trigger.
