@@ -1,6 +1,7 @@
 #include "utilsSystem.h"
 #include "unixutils.h"
 #include "smallUsefulFunctions.h"
+#include <QSvgRenderer>
 
 #ifdef Q_OS_MAC
 #include <CoreFoundation/CFURL.h>
@@ -10,6 +11,7 @@
 #ifdef Q_OS_WIN
 #include <windows.h>
 #endif
+#include <QLibraryInfo>
 bool getDiskFreeSpace(const QString &path, quint64 &freeBytes)
 {
 #ifdef Q_OS_WIN
@@ -21,11 +23,11 @@ bool getDiskFreeSpace(const QString &path, quint64 &freeBytes)
 	freeBytesToCaller.QuadPart = 0L;
 
 	if ( !GetDiskFreeSpaceEx( d, &freeBytesToCaller, NULL, NULL ) ) {
-        delete[] d;
+		delete[] d;
 		qDebug() << "ERROR: Call to GetDiskFreeSpaceEx() failed on path" << path;
 		return false;
 	}
-    delete[] d;
+	delete[] d;
 	freeBytes = freeBytesToCaller.QuadPart;
 	return true;
 #else
@@ -134,20 +136,26 @@ QStringList findResourceFiles(const QString &dirName, const QString &filter, QSt
 	if (!dn.startsWith('/') && !dn.startsWith(QDir::separator())) dn = "/" + dn; //add / at beginning
 	searchFiles << ":" + dn; //resource fall back
 	searchFiles.append(additionalPreferredPaths);
-    searchFiles << QCoreApplication::applicationDirPath() + "/../share/texstudio"; //appimage relative path
+	searchFiles << QCoreApplication::applicationDirPath() + "/../share/texstudio"; //appimage relative path
+    searchFiles << QCoreApplication::applicationDirPath() + "/../usr/share/texstudio"; //go-appimage relative path
 	searchFiles << QCoreApplication::applicationDirPath() + dn; //windows new
-    searchFiles << QCoreApplication::applicationDirPath() + "/"; //windows old
-    searchFiles << QCoreApplication::applicationDirPath() + "/dictionaries/"; //windows new
-    searchFiles << QCoreApplication::applicationDirPath() + "/translations/"; //windows new
-    searchFiles << QCoreApplication::applicationDirPath() + "/help/"; //windows new
-    searchFiles << QCoreApplication::applicationDirPath() + "/utilities/"; //windows new
+	searchFiles << QCoreApplication::applicationDirPath() + "/"; //windows old
+	searchFiles << QCoreApplication::applicationDirPath() + "/dictionaries/"; //windows new
+	searchFiles << QCoreApplication::applicationDirPath() + "/translations/"; //windows new
+	searchFiles << QCoreApplication::applicationDirPath() + "/help/"; //windows new
+	searchFiles << QCoreApplication::applicationDirPath() + "/utilities/"; //windows new
 	// searchFiles<<QCoreApplication::applicationDirPath() + "/data/"+fileName; //windows new
 
 #if !defined(PREFIX)
 #define PREFIX ""
 #endif
 
+#if !defined(CMAKE_INSTALL_DATADIR)
+#define CMAKE_INSTALL_DATADIR ""
+#endif
+
 #if defined( Q_WS_X11 ) || defined (Q_OS_LINUX)
+    searchFiles << PREFIX"/" CMAKE_INSTALL_DATADIR"/texstudio" + dn; //X_11
 	searchFiles << PREFIX"/share/texstudio" + dn; //X_11
 #endif
 #ifdef Q_OS_MAC
@@ -190,18 +198,27 @@ QString findResourceFile(const QString &fileName, bool allowOverride, QStringLis
 		if (s.endsWith('/') || s.endsWith('\\')) searchFiles << s;
 		else searchFiles << s + "/";
 #if defined Q_WS_X11 || defined Q_OS_LINUX || defined Q_OS_UNIX
+    searchFiles << PREFIX"/" CMAKE_INSTALL_DATADIR "/texstudio"; //CMAKE definitin
 	searchFiles << PREFIX"/share/texstudio/"; //X_11
-    searchFiles << QCoreApplication::applicationDirPath() + "/../share/texstudio/"; // relative path for appimage
+	searchFiles << QCoreApplication::applicationDirPath() + "/../share/texstudio/"; // relative path for appimage
+    searchFiles << QCoreApplication::applicationDirPath() + "/../usr/share/texstudio/"; // relative path for appimage
 	if (fileName.endsWith(".html")) searchFiles << PREFIX"/share/doc/texstudio/html/"; //for Debian package
+	searchFiles << PREFIX"/share/doc/texstudio/"; //for Debian package
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+	searchFiles << QLibraryInfo::path(QLibraryInfo::TranslationsPath) + "/"; //for systemwise qt_*.qm
+#else
+	searchFiles << QLibraryInfo::location(QLibraryInfo::TranslationsPath) + "/"; //for systemwise qt_*.qm
+#endif
 #endif
 #ifdef Q_OS_MAC
 	searchFiles << QCoreApplication::applicationDirPath() + "/../Resources/"; //macx
-    searchFiles << QCoreApplication::applicationDirPath() + "/../Resources/html/"; //macx
+	searchFiles << QCoreApplication::applicationDirPath() + "/../Resources/html/"; //macx
 #endif
 	searchFiles << QCoreApplication::applicationDirPath() + "/"; //windows old
 	searchFiles << QCoreApplication::applicationDirPath() + "/dictionaries/"; //windows new
 	searchFiles << QCoreApplication::applicationDirPath() + "/translations/"; //windows new
 	searchFiles << QCoreApplication::applicationDirPath() + "/help/"; //windows new
+	searchFiles << QCoreApplication::applicationDirPath() + "/help/build/html/"; //windows new manual
 	searchFiles << QCoreApplication::applicationDirPath() + "/utilities/"; //windows new
 	// searchFiles<<QCoreApplication::applicationDirPath() + "/data/"; //windows new
 
@@ -237,8 +254,8 @@ QString findResourceFile(const QString &fileName, bool allowOverride, QStringLis
  */
 QString quoteSpaces(const QString &s)
 {
-    if (!s.contains(' ')) return s;
-    return '"' + s + '"';
+	if (!s.contains(' ')) return s;
+	return '"' + s + '"';
 }
 
 
@@ -258,13 +275,13 @@ bool useSystemTheme;
 QString getRealIconFile(const QString &icon)
 {
 	if (icon.isEmpty() || icon.startsWith(":/")) return icon;
-    QStringList suffixList{""};
-    if(darkMode)
-        suffixList=QStringList{"_dm",""};
-    QStringList iconNames = QStringList();
-    for(const QString& suffix : suffixList){
+	QStringList suffixList{""};
+	if(darkMode)
+		suffixList=QStringList{"_dm",""};
+	QStringList iconNames = QStringList();
+	for(const QString& suffix : suffixList){
 
-        QString iconThemeName = "";
+		QString iconThemeName = "";
 
 		if (iconTheme == 0) { // colibre
 			iconThemeName = "colibre";
@@ -278,9 +295,9 @@ QString getRealIconFile(const QString &icon)
 					<< ":/images-ng/" + iconThemeName + "/" + icon + suffix + ".svgz"
 					<< ":/modern/images/" + iconThemeName + "/" + icon + suffix + ".png";
 
-        iconNames << ":/symbols-ng/icons/" + iconThemeName + "/" + icon + suffix + ".svg" ;//voruebergehend
-        iconNames << ":/symbols-ng/icons/" + iconThemeName + "/" + icon + suffix + ".png"; //voruebergehend
-        iconNames << ":/images/" + iconThemeName + "/" + icon + ".png";
+		iconNames << ":/symbols-ng/icons/" + icon + suffix + ".svg";
+		iconNames << ":/symbols-ng/icons/" + icon + suffix + ".png";
+		iconNames << ":/images/" + icon + ".png";
 
 		// fallback
 		iconNames
@@ -289,7 +306,7 @@ QString getRealIconFile(const QString &icon)
     }
 
 	foreach (const QString &name, iconNames) {
-        if (QFileInfo::exists(name))
+		if (QFileInfo::exists(name))
 			return name;
 	}
 
@@ -312,7 +329,7 @@ QIcon getRealIcon(const QString &icon)
 	QString name = getRealIconFile(icon);
 	QIcon ic = QIcon(name);
 	//if(ic.isNull()){
-#if (defined(Q_OS_OSX))
+#if QT_VERSION_MAJOR<6 && defined(Q_OS_OSX)
 	QPixmap pm(32, 32);
 	pm.load(name);
 	ic = QIcon(pm);
@@ -331,7 +348,7 @@ QIcon getRealIcon(const QString &icon)
  */
 QIcon getRealIconCached(const QString &icon, bool forceReload)
 {
-    if (iconCache.contains(icon) && !forceReload) {
+	if (iconCache.contains(icon) && !forceReload) {
 		return *iconCache[icon];
 	}
 	if (icon.isEmpty()) return QIcon();
@@ -353,25 +370,11 @@ QIcon getRealIconCached(const QString &icon, bool forceReload)
 	return *icn;
 }
 
-/*!
- * \brief Tries to determine if the system uses dark mode
- *
- * This function tries to determine if a system uses "dark mode" by looking up general text color and converting it into gray-scale value.
- * A value above 200 (scale is 0 .. 255 ) is considered as light text color on probably dark background, hence a dark mode is detected.
- * This approach is independent on specific on different systems.
- * \return true -> uses dark mode
- */
-bool systemUsesDarkMode(const QPalette &pal)
-{
-    QColor clr=pal.color(QPalette::Text);
-    return qGray(clr.rgb())>200;
-}
-
 bool isFileRealWritable(const QString &filename)
 {
-    if(QFileInfo::exists(filename)){
-        return QFileInfo(filename).isWritable();
-    }
+	if(QFileInfo::exists(filename)){
+		return QFileInfo(filename).isWritable();
+	}
 
 	QFile fi(filename);
 	bool result = false;
@@ -391,7 +394,7 @@ bool isFileRealWritable(const QString &filename)
  */
 bool isExistingFileRealWritable(const QString &filename)
 {
-    return QFileInfo::exists(filename) && isFileRealWritable(filename);
+	return QFileInfo::exists(filename) && isFileRealWritable(filename);
 }
 
 /*!
@@ -511,8 +514,6 @@ QString getRelativeBaseNameToPath(const QString &file, QString basepath, bool ba
 	QStringList basedirs = basepath.split("/");
 	if (baseFile && !basedirs.isEmpty()) basedirs.removeLast();
 	QStringList dirs = path.split("/");
-	//QStringList basedirs = QStringList::split("/", basepath, false);
-	//QStringList dirs = QStringList::split("/", path, false);
 
 	int nDirs = dirs.count();
 
@@ -577,14 +578,15 @@ QString getNonextistentFilename(const QString &guess, const QString &fallback)
 {
 	QFileInfo fi(guess);
 	if (!fi.exists()) return guess;
-	QRegExp reNumberedFilename("(.*[^\\d])(\\d*)\\.(\\w+)");
-	if (!reNumberedFilename.exactMatch(guess)) {
+    QRegularExpression reNumberedFilename("^(.*[^\\d])(\\d*)\\.(\\w+)$",QRegularExpression::UseUnicodePropertiesOption);
+    QRegularExpressionMatch reNumberedFilenameMatch = reNumberedFilename.match(guess);
+    if (!reNumberedFilenameMatch.hasMatch()) {
 		return fallback;
 	}
-	QString base = reNumberedFilename.cap(1);
-	QString ext = reNumberedFilename.cap(3);
-	int num = reNumberedFilename.cap(2).toInt();
-	int numLen = reNumberedFilename.cap(2).length();
+    QString base = reNumberedFilenameMatch.captured(1);
+    QString ext = reNumberedFilenameMatch.captured(3);
+    int num = reNumberedFilenameMatch.captured(2).toInt();
+    int numLen = reNumberedFilenameMatch.captured(2).length();
 
 	for (int i = num + 1; i <= 1000000; i++) {
 		QString filename = QString("%1%2.%3").arg(base).arg(i, numLen, 10, QLatin1Char('0')).arg(ext);
@@ -684,20 +686,20 @@ void showInGraphicalShell(QWidget *parent, const QString &pathIn)
 	const QString folder = fileInfo.isDir() ? fileInfo.absoluteFilePath() : fileInfo.filePath();
 	QSettings dummySettings;
 	const QString app = UnixUtils::fileBrowser(&dummySettings);
-    QProcess browserProc;
-    const QString browserArg = UnixUtils::substituteFileBrowserParameters(app, folder);
+	QProcess browserProc;
+	const QString browserArg = UnixUtils::substituteFileBrowserParameters(app, folder);
 #if QT_VERSION>=QT_VERSION_CHECK(5,15,0)
-    QStringList args=QProcess::splitCommand(browserArg);
+	QStringList args=QProcess::splitCommand(browserArg);
 #else
-    QStringList args=browserArg.split(" "); // this assumes that the command is not using quotes with spaces in the command path, better solution from qt5.15 ...
+	QStringList args=browserArg.split(" "); // this assumes that the command is not using quotes with spaces in the command path, better solution from qt5.15 ...
 #endif
-    if(args.isEmpty())
-        return;
-    QString cmd=args.takeFirst();
-    for(QString &elem:args){
-        elem=removeQuote(elem);
-    }
-    bool success = browserProc.startDetached(cmd,args);
+	if(args.isEmpty())
+		return;
+	QString cmd=args.takeFirst();
+	for(QString &elem:args){
+		elem=removeQuote(elem);
+	}
+	bool success = browserProc.startDetached(cmd,args);
 	const QString error = QString::fromLocal8Bit(browserProc.readAllStandardError());
 	success = success && error.isEmpty();
 	if (!success)
@@ -745,9 +747,9 @@ bool isRetinaMac()
 		process.start("sysctl", QStringList() << "-n" << "hw.model");
 		process.waitForFinished(1000);
 		QString model(process.readAllStandardOutput()); // is something like "MacBookPro10,1"
-		QRegExp rx("MacBookPro([0-9]*)");
-		rx.indexIn(model);
-		int num = rx.cap(1).toInt();
+        QRegularExpression rx("MacBookPro([0-9]*)");
+        QRegularExpressionMatch rxMatch = rx.match(model);
+        int num = rxMatch.captured(1).toInt();
 		if (num >= 10) // compatibility with future MacBookPros. Assume they are also retina.
 			isRetina = true;
 	}
@@ -802,9 +804,24 @@ void SafeThread::wait(unsigned long time)
 QSet<QString> convertStringListtoSet(const QStringList &list)
 {
 #if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
-    return QSet<QString>(list.begin(),list.end());
+	return QSet<QString>(list.begin(),list.end());
 #else
-    return QSet<QString>::fromList(list);
+	return QSet<QString>::fromList(list);
 #endif
 
 }
+/*!
+ * \brief load Pixmap From SVG directly with given target size
+ * \param fn file name
+ * \param sz target size
+ * \return loaded pixmap
+ */
+QPixmap loadPixmapFromSVG(const QString &fn, const QSize sz)
+{
+    QSvgRenderer svgRender(fn);
+    QImage img(sz, QImage::Format_ARGB32);
+    QPainter p(&img);
+    img.fill(0x000000000);
+    svgRender.render(&p);
+    return QPixmap::fromImage(img);
+}
